var MAX_FILES_DROP = 20;
var RENDER_TIMEOUT = 300;
var languageMenu;
var helpMenu;

// View object processes .hbs template
var view = new View({
  'filterKeys': true,
  // 'contextMenuOff' : true
});

view.render = function() {
  $('#content-header').html(
    this.template.contentHeader(model.data)
  );
  $('#notification').html(
    this.template.notification(model.data)
  );
  $('#content-body').html(
    this.template.contentBody(model.data)
  );
  this.hooks.render();
  if (languageMenu !== undefined) {
    languageMenu.destroy();
  }
  if (helpMenu !== undefined) {
    helpMenu.destroy();
  }
  $('.bootstrapMenu').remove();
  languageMenu = buildLanguageMenu();
};

if (!(ruby.isHtmlDialog)) {
  view.addHook('render', checkFlexToolsVersion);
};
view.addHook('render', refreshUpdateUI);
view.addHook('refresh', function() {
  overlay.hide();
  notification.hide();
});
view.addHook('render', function() {
  gettingStartedTooltips.show();
});
view.addHook('render', function() {
  for (var idx in model.data.packages) {
    var pkg = model.data.packages[idx];
    if (pkg.is_demo && pkg.expires_at) {
      var secsLeft = (new Date(pkg.expires_at) - Date.now()) / 1000;
      startCountDown(pkg.id, secsLeft);
    }
  }
});

view.registerHelper('formatCountDown', function(expiresAt) {
  var secsLeft = Math.max(0, (new Date(expiresAt) - Date.now()) / 1000);
  return new Handlebars.SafeString(formatCountDown(secsLeft));
});

// Count Down

function formatCountDown(timeLeft) {
  var days = Math.floor(timeLeft / (24 * 3600));
  var remHours = timeLeft % (24 * 3600);
  var hours = Math.floor(remHours / 3600)
  var remMins = remHours % 3600;
  var mins = Math.floor(remMins / 60)
  var secs = Math.floor(remMins % 60);
  var str = "";
  if (days > 0) {
    str += days + (days > 1 ? " days " : " day ");
  } else {
  str +=
    padStart(2, "0", hours.toString()) +
    ":" +
    padStart(2, "0", mins.toString()) +
    ":" +
    padStart(2, "0", secs.toString());
  }
  return(str);
}

function padStart(n, char, str) {
  var diff = n - str.length;
  if (n > 0) {
    return char.repeat(diff) + str;
  } else {
    return str;
  }
}

var countDownIntervals = {};

function startCountDown(pkgid, startValue) {
  if (countDownIntervals[pkgid]) {
    clearCountDown(pkgid);
  }

  countDownIntervals[pkgid] = {}
  countDownIntervals[pkgid]['value'] = startValue;
  countDownIntervals[pkgid]['intId'] = setInterval(makeCountDownTick(pkgid), 1000);
}

function clearCountDown(pkgid) {
  if (countDownIntervals[pkgid]) {
    clearInterval(countDownIntervals[pkgid]['intId']);
    delete countDownIntervals[pkgid];
  }
}

function makeCountDownTick(pkgid) {
  return function() {
    if (countDownIntervals[pkgid]['value'] >= 1.0) {
      countDownIntervals[pkgid]['value'] -= 1;
    } else {
      countDownIntervals[pkgid]['value'] = 0;
      clearCountDown(pkgid);
    }

    $countDown = $('#count-down-' + pkgid);
    if ($countDown) {
      $countDown.html(formatCountDown(countDownIntervals[pkgid]['value']));
    }
  };
}

// Storage for transfer chosen fpk
// path from ruby
var fpkPath = {
  'value': '',
  'put': function(path) {
    fpkPath.value = path;
  },
  'clear': function() {
    fpkPath.value = '';
  },
  'pop': function() {
    var value = fpkPath.value;
    fpkPath.clear();
    return value;
  }
};

// Overlay used to display message
// to user when plugin is busy or
// when user drags something over
// the window.
var overlay = {
  'active': function(msg) {
    $('#overlay-message').html(msg);
    $('#overlay').attr('class', 'overlay-active');
    return utils.wait(RENDER_TIMEOUT);
  },
  'busy': function(msg) {
    $('#overlay-message').html(msg);
    $('#overlay').attr('class', 'overlay-busy');
    return utils.wait(RENDER_TIMEOUT);
  },
  'hide': function() {
    $('#overlay').attr('class', 'overlay-hidden');
    return utils.wait(RENDER_TIMEOUT);
  }
};

var notification = {
  'hide': function() {
    $('#notification').slideUp(300);
  },
  'show': function(msg) {
    $('#notification-text').html(msg);
    $('#notification').slideDown(300);
  }
};

var gettingStartedTooltips = {
  'show': function() {
    if (
      model.data.options.showGettingStartedTooltips &&
      model.data.allProductsActivated
    ) {
      var btnLearnContent =
        view.template.gettingStartedTooltip({
          'icons': model.data.icons,
          'targetID': '#btn-get-help',
          'tooltipText': model.data.text.get_help
        });
      $('#btn-get-help').popover({
        'placement': 'bottom',
        'content': btnLearnContent,
        'html': true,
        'trigger': 'manual'
      });
      $('#btn-get-help').popover('show');
      view.on('click', '.btn-close-tooltip', function() {
        var targetName = $(this).data('target-id');
        $(targetName).popover('destroy');
      });
      ruby.send('disable_getting_started_tooltips');
    }
  }
}

var updateCheck = {
  'started': function() {
    notification.hide();
    view.refreshElement($('#btn-check-for-updates'), 'btnCheckForUpdates');
  },
  'failed': function() {
    notification.show(model.data.text.update_check_failed);
    view.refreshElement($('#btn-check-for-updates'), 'btnCheckForUpdates');
  },
  'nothing': function() {
    notification.show(model.data.text.update_check_nothing);
    view.refreshElement($('#btn-check-for-updates'), 'btnCheckForUpdates');
  },
  'expired': function(pkgName) {
    notification.show(utils.str_format(model.data.text.update_check_expired, pkgName));
    view.refreshElement($('#btn-check-for-updates'), 'btnCheckForUpdates');
  }
}

function activate(id) {
  var dfrActivation = $.Deferred();
  var inputField = $('#serial-key-' + id);
  // Remove trailing spaces at the begging and the end of serial key
  var serialKey =
    inputField.val()
              .replace(/^\ +([1-9a-zA-Z\-]*)\ +$/,'$1');
  if (serialKey === undefined || serialKey === '') {
    inputField.focus();
  } else {
    overlay.busy(model.data.text.manager_activating_product)
      .then(function() {
        return ruby.send('activate', [{'serial_key': serialKey, 'package_id': id}]);
      })
      .done(function() {
        dfrActivation.resolve();
      });
  }
  return dfrActivation;
}

function checkFlexToolsVersion() {
  if (model.data.versions.checkLatest) {
    $.post(
      model.data.versions.uri,
      function(update_info_json) {
        if (update_info_json.status) {
          ruby.send('save_versions_info', [JSON.stringify(update_info_json.message)])
              .done(function() {
                refreshUpdateUI(true);
              });
        }
      }
    );
  }
}

function refreshUpdateUI(fetchData) {
  if (fetchData === true) {
    model.fetchData().done(function() {
      refreshUpdateUI();
    });
  }
}

function openUrl(url) {
  return ruby.call('open_url', [url]);
}

function toggleSettings(pid) {
  return ruby.send('toggle_settings', [pid]);
}

function toggleViewMode() {
  ruby.send('toggle_view_mode')
    .done(function() {
      view.refresh();
    });
}

function showAbout() {
  return ruby.send('show_about');
}

function invokeItemAction(pid) {
  return ruby.send('invoke_item_action', [pid]);
}

function setLocale(lang) {
  return ruby.send('set_locale', [lang]);
}

// Adds delay to BootstrapMenu
// before closing it.
function addDelayBeforeClose(_menu) {
  var menuCloseDfr;
  var clearCloseEventListeners = function(menu) {
    menu.$openTarget.off(menu.closeNamespace);
    menu.$menu.off(menu.closeNamespace);
  };
  var addDelayedCloseEventListeners = function(menu) {
    var closeEventListener = function() {
      closeWithDelay(menu);
    };
    menu.$openTarget.on('mouseleave' + menu.closeNamespace, closeEventListener);
    menu.$menu.on('mouseleave' + menu.closeNamespace, closeEventListener);
    addDfrRejectListeners(menu);
  };
  var closeWithDelay = function(menu) {
    if (menu.$menu.is(':hidden')) {
      return;
    }
    menuCloseDfr = utils.wait(800);
    menuCloseDfr.done(function() {
      menu.close();
    });
  };
  var addDfrRejectListeners = function(menu) {
    menu.$openTarget.on('mouseenter' + menu.closeNamespace, function() {
      menuCloseDfr.reject();
    });
    menu.$menu.on('mouseenter' + menu.closeNamespace, function() {
      menuCloseDfr.reject();
    });
    menu.$menu.on('click', function() {
      menuCloseDfr.reject();
    });
  };
  // Extending menu.open method:
  _menu.open = function($openTarget, ev) {
    BootstrapMenu.prototype.open.call(_menu, $openTarget, ev);
    clearCloseEventListeners(_menu);
    addDelayedCloseEventListeners(_menu);
  };
}

function buildLanguageMenu() {
  if (model.data.locale.all.length === 0) {
    return;
  }
  var localeFullName = function(locale) {
    if (locale.name !== null) {
      return locale.name + ' (' + locale.code + ')';
    } else {
      return locale.code;
    }
  };
  var autoAction = {
    'name': 'Automatic',
    'isEnabled': function() {
      return model.data.locale.current !== 'automatic';
    },
    'onClick': function() {
      setLocale('automatic');
    }
  };
  var localeActions = $.map(model.data.locale.all, function(locale) {
    return {
      'name': localeFullName(locale),
      'isEnabled': function() {
        return locale.code !== model.data.locale.current;
      },
      'onClick': function() {
        setLocale(locale.code);
      }
    };
  });
  var allActions = [autoAction].concat(localeActions);
  var menu = new BootstrapMenu('#btn-locale', {
    menuEvent: 'click',
    menuSource: 'element',
    menuPosition: 'belowRight',
    actions: allActions
  });
  addDelayBeforeClose(menu);
  return menu;
}

function installByPath(path) {
  ruby.send('install_by_path', [path])
    .done(function() {
      view.refresh();
    });
}

function installByDrop() {
  ruby.send('install_by_drop')
    .done(function(){
      view.refresh();
    });
}

function installDropped(index, fpkList) {
  var fpkId, fpkData, fpkField;
  if (index === fpkList.length) {
    installByDrop();
  } else {
    var file = fpkList[index];
    var reader = new FileReader();
    reader.onload = function(ev) {
      if (ruby.isHtmlDialog) {
        fpkId = 'fpk-data-' + index;
        fpkData = file.name + ':::' + ev.target.result;
        ruby.call('store_file', [fpkId, fpkData])
          .done(function() {
            installDropped(index + 1, fpkList);
          });
      } else {
        fpkId = 'fpk-data-' + index;
        //Saving file data to fpk-data-* field.
        fpkField = $('#' + fpkId);
        fpkData = file.name + ':::' + ev.target.result;
        fpkField.val(fpkData);
        ruby.call('store_file', [fpkId])
          .done(function() {
            installDropped(index + 1, fpkList);
          });
      }
    };
    reader.readAsDataURL(file);
  }
}

function dropHandler(ev) {
  ev.stopPropagation();
  ev.preventDefault();
  var files = ev.originalEvent.dataTransfer.files;
  if (files.length >= MAX_FILES_DROP) {
    utils.messagebox(model.data.text.too_many_files);
    return;
  }
  var fpkList = [];
  for (var i = 0; files[i]; i++) {
    var f = files[i];
    // Checking if dropped files have .fpk or .rbz extension.
    var ext = f.name.split('.').pop();
    if (ext === 'fpk' || ext === 'rbz') {
      fpkList.push(f);
    } else {
      // Notify user about ignored file.
      var errMsg =
        model.data.text.error + ': ' + f.name +
        model.data.text.is_not_fpk;
      utils.messagebox(errMsg);
    }
  }
  switch(fpkList.length) {
  case 0:
    overlay.hide();
    return;
  case 1:
    overlay.busy(model.data.text.manager_installing + ' ' + fpkList[0].name)
      .done(function() {
        installDropped(0, fpkList);
      });
    return;
  default:
    overlay.busy(utils.str_format(model.data.text.manager_installing_n_packages, [fpkList.length]))
      .done(function() {
        installDropped(0, fpkList);
      });
    return;
  }
}

var dragHandler = {
  //This counter is workaround to prevent
  //overlay flickering resulting from drag
  //event catches by body children elements.
  'counter': 0,
  'enter': function(ev) {
    ++(dragHandler.counter);
    overlay.active('+ ' + model.data.text.manager_install_packages);
    ev.stopPropagation();
    ev.preventDefault();
  },
  'leave': function(ev) {
    --(dragHandler.counter);
    if (dragHandler.counter === 0) {
      overlay.hide();
    }
    ev.stopPropagation();
    ev.preventDefault();
  },
  'over': function(ev) {
    ev.stopPropagation();
    ev.preventDefault();
    ev.dataTransfer.dropEffect = 'copy';
  }
};

view.on('click', '.btn-ft-link', function() {
  openUrl('https://flextools.cc');
});

view.on('click', '#btn-get-help', function() {
  openUrl('https://flextools.cc/resources/');
});

view.on('click', '#btn-hide-notification', function() {
  notification.hide();
});

view.on('click', '#btn-toggle-view-mode', toggleViewMode);

view.on('click', '#btn-install', function() {
  ruby.call('open_archives')
});

view.on('click', '.btn-activate', function() {
  var pid = $(this).val();
  activate(pid);
    // .done(function() {
    // view.refresh();
    // });
});

view.on('click', '.btn-visibility', function() {
  var btn = $(this);
  var args = btn.val().split(';');
  var id = args[0];

  ruby.send('toggle_toolbar_visibility', [id])
    .done(function() {
      model.fetchData()
        .done(function() {
          var isTarget = function (entity) { return entity.id == id }
          var newTarget = model.data.components.find(isTarget) || model.data.tools.find(isTarget);
          view.refreshElement(btn, 'btnVisibility', newTarget);
          notification.show(model.data.text.restart_sketchup_to_rebuild_roolbar);
        })
    });
});

view.on('click', '.btn-settings', function() {
  var pid = $(this).val();
  toggleSettings(pid);
});

view.on('click', '.btn-info-url', function() {
  var url = $(this).val();
  openUrl(url);
});

view.on('click', '.btn-product-thumb', function() {
  var pid = $(this).val();
  invokeItemAction(pid);
});

view.on('click', '#btn-toggle-presets-dialog', function() {
  ruby.send('toggle_presets_dialog');
});

view.on('click', '#btn-toggle-auto-rotator', function() {
  var componentList = $('#component-list');
  ruby.send('toggle_auto_rotator')
    .done(function() {
      view.refreshElement(componentList, 'componentList');
    });
});

view.on('click', '#btn-toggle-wallcutter', function() {
  var componentList = $('#component-list');
  ruby.send('toggle_wallcutter')
    .done(function() {
      view.refreshElement(componentList, 'componentList');
    });
});

view.on('click', '.btn-update', function() {
  ruby.send('update');
});

view.on('click', '.btn-release-notes', function() {
  openUrl('https://flextools.cc/release-notes/');

});

view.on('click', '.btn-check-for-updates', function() {
  ruby.send('sync')
    .done(function() {
      view.refresh();
    });
});

view.on('click', '#btn-about', showAbout);

view.on('click', 'a', function() {
  var url = $(this).attr('value');
  openUrl(url);
});

view.on('click', '.btn-license', function () {
  var pkgId = $(this).val();
  ruby.send('show_package_license', [pkgId]);
});

view.on('click', '.btn-package-uninstall', function () {
  var pkgId = $(this).val();
  ruby.send('uninstall_package', [pkgId]);
});

view.on('keydown', '.form-control-activation', function(ev) {
  var ENTER_CODE = 13;
  var keyCode = ev.which;
  if (keyCode === ENTER_CODE) {
    var pid = $(this).attr('data-pid');
    activate(pid);
      // .done(function() {
      //   view.refresh();
      // });
  }
});

// Resizing singleton activation input field when it focused
$(document).on(
  'focusin',
  '.controls-singleton * .form-control-activation',
  function() {
    var itemControls = $(this).parents('.item-controls')[0];
    var controlsRight = $(itemControls).find('.controls-right');
    $(controlsRight).hide();
    var controlsLeft = $(this).parents('.controls-singleton')[0];
    $(controlsLeft).removeClass('controls-short').addClass('controls-long');
  }
);

// Returning singleton activation input field back to unfocused size
$(document).on(
  'focusout',
  '.controls-singleton > .input-group-activation',
  function() {
    var cursorHover = $(this).is(':hover');
    if (!cursorHover) {
      var controlsLeft = $(this).parents('.controls-singleton')[0];
      $(controlsLeft).removeClass('controls-long').addClass('controls-short');
      var itemControls = $(this).parents('.item-controls')[0];
      var controlsRight = $(itemControls).find('.controls-right');
      $(controlsRight).show();
    }
  }
);

//Setup the drag and drop listeners.
$(document).on('dragenter', dragHandler.enter);
$(document).on('dragleave', dragHandler.leave);
$(document).on('dragover',  dragHandler.over);
$(document).on('drop', dropHandler);

view.refresh();
