function View(parameters) {
  // Declaring fields
  // Boolean flags list
  this.flags = {};
  // Handlebars templates list
  this.template = {};
  // Key callbacks
  this.keyActions = {};
  // List of hooks
  this._hooks = {};
  // Hooks runner
  this.hooks = {};
  // Pointer position
  this.pointerX = 0;
  this.pointerY = 0;
  // Copying flags from parameters
  if (parameters) {
    this.flags.filterKeys = parameters.filterKeys;
    this.flags.contextMenuOff = parameters.contextMenuOff;
  }
  // Assign default keyActions values
  // ESC key
  this.keyActions[27] = function () {
    this.refresh();
  }.bind(this);
  // Rewriting keys callbacks
  if (parameters && parameters.keyActions) {
    $.each(parameters.keyActions, function (key, callback) {
      this.keyActions[key] = callback;
    }.bind(this));
  }
  // Calling initializer
  this.init.call(this);
}

View.prototype.init = function() {
  // Runs render hooks
  this.hooks.render = function() {
    $.each(this._hooks.render, function(_, hook) {
      hook();
    });
  }.bind(this);

  //Runs refresh hooks
  this.hooks.refresh = function() {
    $.each(this._hooks.refresh, function(_, hook) {
      hook();
    });
  }.bind(this);

  // Blocking some keys to provide
  // more native look and feel
  var filterKeys = function() {
    // List of keys user can use
    // with ctrl modifier
    var keyCtrlWhiteList = {
      8:  'BCKSP',
      37: 'LEFT',
      65: 'A',
      67: 'C',
      39: 'RIGHT',
      86: 'V',
      90: 'Z',
      88: 'X'
    };
    // List of keys blocked
    var keyBlackList = {
      9: 'TAB',
      116: 'F5'
    };
    // Blocking keys using black/white lists.
    $(document).on('keydown', function (e) {
      if (keyBlackList[e.which]) {
        return false;
      }
      if (e.ctrlKey === true) {
        if (!keyCtrlWhiteList[e.which]) {
          return false;
        }
      }
    });
    $(document).on('mousewheel', function (e) {
      // Disable zooming with mouse wheel
      if (e.ctrlKey === true) {
        return false;
      }
    });
  };

  var contextMenuOff = function contextMenuOff() {
    $(document).on('contextmenu', '*', function(e) {
      if ($(this).hasClass('allow-context-menu')) {
        e.stopPropagation();
      } else {
        e.preventDefault();
      }
    });
  };

  var setPointerPosition = function setPointerPosition(x, y) {
    this.pointerX = x;
    this.pointerY = y;
  }.bind(this);

  if (this.flags.filterKeys) {
    filterKeys();
  }

  if (this.flags.contextMenuOff) {
    contextMenuOff();
  }

  $(document).mousemove(function(ev) {
    setPointerPosition(ev.pageX, ev.pageY);
  });

  // Processing special keys
  $(document).on('keydown', function(ev) {
    var keyCode = ev.which;
    if (this.keyActions[keyCode]) {
      this.keyActions[keyCode](ev);
    }
  }.bind(this));
  // This hook fixes tooltips for HtmlDialog
  if (ruby.isHtmlDialog && ruby.suMajorVersion == 17) {
    var tooltipTimeoutDefer;

    var isInDom = function($elem) {
      return $elem.closest('body').length > 0;
    };

    var showTooltip = function() {
      var $titleElement = $(this);
      var tooltip = '<div id="tooltip-fallback" class="tooltip-fallback" hidden>' +
                    $titleElement.attr('title') +
                    '</div>';
      tooltipTimeoutDefer = utils.wait(800);
      tooltipTimeoutDefer.done(function() {
        if (isInDom($titleElement)) {
          $('.tooltip-fallback').remove();
          $('body').append(tooltip);
          var tooltipElem = $('#tooltip-fallback');
          var windowWidth = $(window).width();
          var windowHeight = $(window).height();
          var cursorHeight = 15;
          var widthThreshold = 50;
          tooltipElem.css('top',  this.pointerY);
          var width = tooltipElem.width();
          var tooltipLeftPos;
          if (this.pointerX + width + widthThreshold <= windowWidth) {
            tooltipLeftPos = this.pointerX;
          } else {
            tooltipLeftPos = windowWidth - (width + widthThreshold);
          }
          var tooltipTopPos;
          tooltipElem.css('left', tooltipLeftPos);
          if (this.pointerY + cursorHeight >= windowHeight) {
            tooltipTopPos = this.pointerY - tooltipElem.height();
          } else {
            tooltipTopPos = this.pointerY + tooltipElem.height() + cursorHeight;
          }
          tooltipElem.css('top',  tooltipTopPos);
          tooltipElem.fadeIn(300);
        } else {
          hideTooltip();
        }
      }.bind(this));
    };

    var hideTooltip = function() {
      tooltipTimeoutDefer.reject();
      var $tooltipElement = $('#tooltip-fallback');
      $tooltipElement.fadeOut(150, function() {
        $('.tooltip-fallback').remove();
      });
    };

    var hoverFallbackHook = function() {
      var elementsWithTitle = $('[title]');
      $.each(elementsWithTitle, function(_, elem) {
        var $elem = $(elem);
        if ($elem.data('hasHoverListener') === undefined) {
          $elem.hover(showTooltip, hideTooltip);
          $elem.click(hideTooltip);
          $elem.data('hasHoverListener', true);
        }
      });
    };

    this.addHook('render', hoverFallbackHook);
  }

  var templates = $('[type="text/x-handlebars-template"]');
  $.each(templates, function(_, template) {
    var templateNameKebabCase =
      template.id.replace(/-template/,'');
    var templateNameKebabCamelCase =
      templateNameKebabCase.replace(/[-](.)/g, function(v) {
        return v.toUpperCase();
      });
    var templateJsName =
      templateNameKebabCamelCase.replace(/-/g, '');
    this.addTemplate(templateJsName, template.id);
  }.bind(this));
};

View.prototype.addTemplate = function(templateName, elementID) {
  this.template[templateName] = HandlebarsExt.compileTemplate(elementID);
  Handlebars.registerPartial(templateName, this.template[templateName]);
};

View.prototype.refresh = function() {
  model.fetchData().done( this.render.bind(this) );
  this.hooks.refresh();
};

View.prototype.refreshElement = function(elem, templateName, _ctx) {
  var refreshTemplate = function refreshTemplate() {
    var ctx;
    if (_ctx === undefined) {
      ctx = model.data;
    } else {
      ctx = _ctx;
      ctx.data = model.data;
    }
    var template = this.template[templateName];
    elem.replaceWith(template(ctx));
    this.hooks.render();
  }.bind(this);

  model.fetchData()
    .done(refreshTemplate);
};

View.prototype.render = function() {
  $('#content').html(
    this.template.content(model.data)
  );
  this.hooks.render();
};

View.prototype.addHook = function(type, hook) {
  if (this._hooks[type] === undefined) {
    this._hooks[type] = [];
  }
  this._hooks[type].push(hook);
};

View.prototype.registerHelper = function(helperName, helperCallback) {
  Handlebars.registerHelper(helperName, helperCallback);
};

View.prototype.on = function(eventName, selector, callback) {
  $(document).on(eventName, selector, callback);
};
